package com.ibm.bsf.engines.rexx;

import  java.lang.reflect.*;

/**
 * This class serves as a wrapper for Java arrays. Its methods allow to fully interact
 * with arrays of any type from Rexx (and other non-Java-based languages). Names of
 * methods are called after their Object Rexx counterparts, if any.
 *
 * @author Rony G. Flatscher
 * @version 0.9.0, 2003-02-09
 * @since   2003-02-09 by rgf
 */

public class ArrayWrapper {


    /** This hashtable serves as a registry for arrays already analyzed by an instance of this class.
     */
    static protected java.util.Hashtable array2wrapper = new java.util.Hashtable();


    /** Stores the array object which this instance wraps. */
    private Object arrayObject=null;


    /** Stores the total number of dimensions. */
    public int dimensions=0;

    /** Returns the size of the given dimension.
     *
     * @param dimension indicates the dimension of interest (Java index, i.e. first dimension to be
     *        indicated/indexed with the value <pre>0</pre>).
     *
     * @return the size of the given dimension.
     */
    public int dimension (int i) {
        return sizeOfDimension[i];
    }

    /** Array of <pre>int</pre> which stores the size (maximum number of entries) of the appropriate dimension. */
    public int [] sizeOfDimension=null;


    /** Stores the total size (maximum number of entries) of the entire array. */
    public int items=1;     // must start with one (will be multiplied with the size of the dimensions


    /** Stores the class object used for the values stored in the array. */
    public Class componentType=null;

    /** Only method to get an instance of ArrayWrapper created. If there is already
     *  an instance for the given array in the Hashtable {@link #array2wrapper}, that
     *  instance is returned, else an instance is created, stored in the Hashtable and
     *  then returned.
     *
     *  @param objArray the array object for which an ArrayWrapper instance is sought
     *
     *  @return an ArrayWrapper instance.
     */
    static public ArrayWrapper getArrayWrapper (Object objArray)
    {
        String str=objArray.getClass().getName() + '@' + Integer.toHexString( objArray.hashCode() );
// System.err.println("getArrayWrapper(): received <"+objArray+"> == self made: <"+str+">");

        ArrayWrapper aw=(ArrayWrapper) array2wrapper.get(str); // try to get the object

// System.err.println("aw="+ (aw==null ? "null" : aw.toString()));
        if (aw==null)       // o.k., not stored yet
        {
            aw=new ArrayWrapper(objArray);  // create an instance
            array2wrapper.put(str, aw);     // now store the new instance
        }
        return aw;
    }



    /** A private constructor. Gets used if there is no ArrayWrapper instance in {@link #array2wrapper}. */
    private ArrayWrapper (Object objArray)
    {
        arrayObject=objArray;   // save the array with this instance of ArrayWrapper

// System.err.print("ArrayWrapper()... ");
        int    i=0, len=0;

            // determine total number of dimensions
        String className=objArray.getClass().getName();   // get name of class
        len=className.length();

        for (; i<len; i++)
        {
            if ('['==className.charAt(i))
            {
                dimensions++;
            }
            else break;      // no more leadin square brackets left
        }

        char elementType=className.charAt(i);   // get the element type letter
        switch (elementType)
        {       // primitive component types
            case 'I':   componentType=int.class;       break;
            case 'C':   componentType=char.class;      break;
            case 'Z':   componentType=boolean.class;   break;
            case 'B':   componentType=byte.class;      break;
            case 'D':   componentType=double.class;    break;
            case 'F':   componentType=float.class;     break;
            case 'S':   componentType=short.class;     break;
            case 'J':   componentType=short.class;     break;

                // Object component type
            case 'L':   {       // extract name of class
                            String tmp="";
                            int    end=len-1;
                            for (i++; i<end; i++)
                            {
                                tmp=tmp+className.charAt(i);
                            }
                            try
                            {
                                componentType=Class.forName(tmp);   // get appropriate class object
                            }
                            catch (Exception e)
                            {
                                e.printStackTrace();
                                // throw e;
                            }
// System.err.println("===> tmp=["+tmp+"]");
                        }
                        break;
        }

// System.err.println("===> elementType="+elementType+", componentType="+componentType);

/*
 Lclassname;  class or interface

     I            int
     C            char
     Z            boolean
     B            byte
     D            double
     F            float
     S            short
     J            long
*/


            // determine the size of the individual dimensions and total entries/items
       sizeOfDimension = new int [dimensions];    // sizeOfDimension array

       Object tmpArray = objArray;  // assign passed in array to tmpArr variable

        // calculate the total number of elements needed
       for (i=0; i<dimensions; i++)    // loop over all dimensions/arrays
       {
           sizeOfDimension[i] = Array.getLength(tmpArray);// save the length of this dimension
           items *= sizeOfDimension[i];                 // total number of possible elements
           tmpArray=Array.get(tmpArray, 0);   // get next array, if any
       }
// System.err.println(" dimensions="+dimensions+", items="+items);

    }




    /** Get a value (item, element) from the array at the given position.
     *
     * @param idxArr a one-dimensional array of <pre>int</pre>s determining the location in the array.
     *
     * @return the Object stored at the specified location.
    */

    public Object get(int[] idxArr)
    {

// System.err.println("in ArrayWrapper.get(): dimensions="+dimensions+", idxArr.length="+idxArr.length+"...");
/*
for (int i=0; i<idxArr.length; i++)
{
    System.err.print("\t\tidxArr["+i+"]=");
    System.err.println(idxArr[i]);
}
*/

        int i
            // dimensions=rgfGetDimension(objArr)  // get the dimension of this array
            ;
        Object o=null;

        o=arrayObject;
        // for (i=0; i<idxArr.length; i++)
        for (i=0; i<dimensions && i<idxArr.length; i++)
        {
            o=Array.get( o, idxArr[i] );
// System.err.println("\tarrayObject="+arrayObject+", i="+i+", idxArr[i]="+idxArr[i]+": o="+o);
        }
// System.err.println("\tarrayObject, returning: "+o);
        return o;
    }



    /** Put a value (item, element) into the array object at the given position.
     *
     * @param idxArr a one-dimensional array of <pre>int</pre>s determining the location in the array.
     *
     * @param valueObj the Object to store; in the case of primitives the appropriate conversions
     *                 takes place, curtesy of {@link java.lang.reflect.Array}.
     */
    public void put(Object valueObj, int [] idxArr)
    {

// System.err.println("in ArrayWrapper.put()...");

        int i,
            // upperLimit=rgfGetDimension(objArr)-1
            upperLimit=dimensions-1
        ;

        Object o=null;

        o=arrayObject;
        for (i=0; i<upperLimit && i<idxArr.length; i++)     // find appropriate array object first
        {
            o=Array.get( o, idxArr[i] );
// System.err.println("/// arrayObject="+arrayObject+", i="+i+", idxArr[i]="+idxArr[i]+": o="+o+", idxArr.getClass()="+idxArr.getClass());
        }

            // now assign the value
// System.err.println("/// set: o="+o+", idxArr[i]="+idxArr[i]+", valueObj="+valueObj);
       Array.set(o, idxArr[i], valueObj);
   }




    /** Creates a supplier object for allowing to enumerate the array together with
     *  the appropriate indices (Java style: index starts with <pre>0</pre> and is
     *  enclosed in square brackets).
     *
     *  @return an instance of class {@link Supplier}, allowing for enumerating the contents of the array.
     */
    public Supplier supplier ()
    {
        Object [] res=makearray(true, false);
        return new Supplier( (Object []) res[0], (Object []) res[1]);  // return the appropriate supplier object
    }


    /** Creates a supplier object for allowing to enumerate the array together with
     *  the appropriate indices.
     *
     *  @param bRexxStyle determines whether the index value is [Object] Rexx
     *         style (index starts with <pre>1</pre>, multiple indices are separated by a comma)
     *         or Java style (index starts with <pre>0</pre>, indices are enclosed in square brackets)
     *
     *  @return an instance of class {@link Supplier}, allowing for enumerating the contents of the array.
     */
    public Supplier supplier (boolean bRexxStyle)
    {
        Object [] res=makearray(true, bRexxStyle);
        return new Supplier( (Object []) res[0], (Object []) res[1]);  // return the appropriate supplier object
    }




    /** Modelled after Object Rexx <pre>MAKEARRAY</pre> method of the array class. This
     *  method is executed <em>synchronized</em>.
     *
     * @param bSupplier boolean value to indicate whether a {@link Supplier supplier} array (containing the appropriate indices as Strings) is to be prepared and returned (if set to <pre>true</pre>).
     *
     * @param bRexxStyle boolean value to indicate whether using Rexx (index starts at: <pre>1</pre>) or Java (index starts at: <pre>0</pre>) style for creating the indices for a {@link Supplier supplier}.
     *
     * @return a two-dimensional array of Objects, where the first element contains the single dimensioned array and
     *         the second element the optional supplier array
     */
    synchronized public Object [] makearray ( boolean bSupplier, boolean bRexxStyle )
    {
         // initialize ...
        int i=0, k=0, m=0,
            idx=0,
            totalCount=items;       // totalCount already calculated

        int    run[] = new int [dimensions];    // run array

        String info="", infoTail="";     // info String containing the index/indices
        Object o=null, tmpArr = arrayObject;  // assign passed in array to tmpArr variable

        Object [] resObj= {null, null},  // single-dimensioned arrays; result Object
                  newArr=null,           // new single-dimensioned array
                  newSupplier=null;      // new single-dimensioned array for supplier

        if (totalCount==0)
        {
            System.err.println("shit, now it has happened! (totalCount=0, array has no capacity, why? think about it...)");
        }

        newArr = new Object [totalCount];   // create array with needed number of elements
        if (bSupplier)                      // supplier really requested, create the array
        {
            newSupplier = (Object[]) new Object [totalCount];
        }


             // loop and walk the array
        while ( run[0]<sizeOfDimension[0] )
        {
 // System.err.println(">>> run[0]="+run[0]);
            tmpArr=arrayObject;       // (re-)assign master array
            k=dimensions-1;             // get array with values
            for (i=0; i<k; tmpArr=Array.get(tmpArr, run[i++]));

            info="";             // create info-string used as value for the supplier-array
            for (int o2=0; o2<(dimensions-1); o2++)
            {
                if (bRexxStyle)          // Object Rexx type array, index starts with '1'
                    info=info+(run[o2]+1)+",";
                else                     // Java type array, index starts with '0'
                    info=info+"["+run[o2]+"]";
            }

            for (i=0; i<sizeOfDimension[k]; i++)  // loop over value-array and assign newArr, if element not null
            {
                o=Array.get(tmpArr, i);    // get value object
                if (o==null) continue;   // empty element, ignore (only possible if an array of Object

                if (bRexxStyle)          // Object Rexx type array, index starts with '1'
                    infoTail=""+(i+1);
                else                     // Java type array, index starts with '0'
                    infoTail="["+i+"]";

// System.err.println("\t\t>>> "+info+infoTail+"\t >>>> "+o);

                if (bSupplier) newSupplier [idx] = info+infoTail;
                newArr[idx++] = o;           // save element in new array
            }

             // increment index (last dimension, carry over to the left)
            for (i=dimensions-2; i>=0; i--)         // increment the next index from right to left
            {
                run[i]=run[i]+1;             // increment present dimension

                if (run[i]<sizeOfDimension[i]) break; // o.k. not yet at peak, incrementation went o.k.

                 // arrived at limit, increase previous (left-hand sided) dimension by 1
                for (int z=i-1; z>=0; z--)
                {
                    run[z]=run[z]+1;         // increase
                    if (run[z]<sizeOfDimension[z])    // o.k. within limits, reset everything right to 0
                    {
                        for (m=z+1; m<dimensions; m++)  // loop over all of the right dimensions
                        {
                            run[m]=0;        // reset the dimension
                        }
                        break; // o.k. this dimension could get increased within limits, leave array
                    }
                }
                break;
            }

            if (dimensions==1) break;       // o.k., just one dimension, leave loop
        }


        if (totalCount != idx)                   // only create a new array, if dimensions differ
        {
// System.err.println("... totalCount != idx (" + totalCount + "!=" +idx+ "), hence creating a new one.");

            Object [] resArr = new Object [idx], // create an array with the right dimensions
                      resSupplier = null;        // create an array variable
            if (bSupplier)
                resSupplier = new Object [idx];  // create the array for the supplier now

            for (i=0; i<idx; i++)                // loop over all elements
            {
                resArr[i]=newArr[i];             // assign values to new array
                if (bSupplier)
                    resSupplier[i]=newSupplier[i];
            }
            newArr=resArr;
            newSupplier=resSupplier;
        }

        resObj[0]=newArr;                    // use the working array as result
        resObj[1]=newSupplier;               // return the supplier array object

        return resObj;
    }

}



